@@ -16,8 +16,9 @@
  *                                                                           *
  *  You should have received a copy of the GNU Library General Public        *
  *  License along with this program; see the file COPYING.LIB.  If not,      *
- *  write to the Free Software Foundation, Inc., 59 Temple Place -           *
- *  Suite 330,  Boston, MA 02111-1307, USA.                                  *
+ *  write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA  02110-1301  USA
  *                                                                           *
  *****************************************************************************/
 
@@ -52,6 +53,8 @@
      0x03 = Endcode   for a command
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -180,6 +183,8 @@ hp_gen_cmd_blob (enum hp215_cmd cmd, int bytes, unsigned char *argdata, unsigned
 	(*buf)[1] = cmd;
 	if (bytes >= 0x7d) {
 		gp_log (GP_LOG_ERROR, "hp215", "Using too large argument buffer %d bytes", bytes);
+		free (*buf);
+		*buf = NULL;
 		return GP_ERROR_BAD_PARAMETERS;
 	}
 	/* store arglen */
@@ -252,7 +257,7 @@ hp_rcv_ack (Camera *cam, char *ackval)
 static int
 decode_u32(unsigned char **msg, int *msglen, unsigned int *val) {
 	unsigned int x = 0,i;
-
+	*val = 0;
 	for (i=0;i<8;i++) {
 		if (!*msglen)
 			return GP_ERROR;
@@ -268,7 +273,7 @@ decode_u32(unsigned char **msg, int *msglen, unsigned int *val) {
 static int
 decode_u16(unsigned char **msg, int *msglen, unsigned short *val) {
 	unsigned int x = 0,i;
-
+	*val = 0;
 	for (i=0;i<4;i++) {
 		if (!*msglen)
 			return GP_ERROR;
@@ -393,6 +398,7 @@ hp_get_timeDate_cam (Camera *cam, char *txtbuffer, size_t txtbuffersize)
 		return ret;
 	xmsg = msg;
 	if (msglen < 0x59) {
+		free (msg);
 		gp_log (GP_LOG_ERROR, "hp215", "too short reply block, %d bytes", msglen);
 		return GP_ERROR_IO;
 	}
@@ -437,7 +443,7 @@ hp_get_timeDate_cam (Camera *cam, char *txtbuffer, size_t txtbuffersize)
 	free (msg);
 
 
-	snprintf (txtbuffer, txtbuffersize, _("Current camera time:  %04d-%02d-%02d  %02d:%02d\nFree card memory: %d\nImages on card: %d\nImages free on card: %d\nBattery level: %d %%"),
+	snprintf (txtbuffer, txtbuffersize, _("Current camera time:  %04d-%02d-%02d  %02d:%02d\nFree card memory: %d\nImages on card: %d\nFree space (Images): %d\nBattery level: %d %%."),
 		date.year, date.month, date.day, date.hour, date.min,
 		freespace, usedimages, freeimages, percent
 	);
@@ -558,7 +564,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       CameraFileInfo *info, void *data, GPContext *context)
 {
 	Camera		*camera = data;
-	int		ret, msglen, buflen, offset = 13, image_no;
+	int		ret, msglen, buflen, /* offset = 13, */ image_no;
 	unsigned int	val, retcode;
 	unsigned char	*xmsg, *msg, *buf;
 
@@ -574,8 +580,11 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	free (buf);
 	if (ret < GP_OK)
 		return ret;
-	if (msglen < 2)
+
+	if (msglen < 2) {
+		free (msg);
 		return GP_ERROR_IO;
+	}
 
 /*
 0000  80 80 80 85 86 85-84 84 30 31 2f 30 32 2f  ..........01/02/
@@ -585,8 +594,10 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
  */
 	xmsg = msg;
 	ret = decode_u32(&xmsg, &msglen, &val);
-	if (ret < GP_OK)
+	if (ret < GP_OK) {
+		free (msg);
 		return ret;
+	}
 	memset (info, 0, sizeof(*info));
 	info->file.fields = GP_FILE_INFO_SIZE;
 	info->file.size = val;
@@ -599,8 +610,10 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	msglen	-= 2;
 
 	ret = decode_u32(&xmsg, &msglen, &val);
-	if (ret < GP_OK)
+	if (ret < GP_OK) {
+		free (msg);
 		return ret;
+	}
 	info->preview.fields = GP_FILE_INFO_SIZE;
 	info->preview.size = val;
 
@@ -653,6 +666,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	return gp_list_populate(list, "image%i.jpg", count);
 }
 
+#if 0 /* This function is not used by anybody */
 static int
 get_shoot_mode_table (Camera *camera) {
 	unsigned char *buf, *msg, *xmsg;
@@ -720,6 +734,7 @@ get_shoot_mode_table (Camera *camera) {
 	free (msg);
 	return GP_OK;
 }
+#endif
 
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
