@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <gphoto2/gphoto2.h>
 
 #include "dc210.h"
@@ -165,7 +166,7 @@ static int dc210_write_single_char
 	int i;
 
 	for (i=0; i< RETRIES; i++){
-		if (gp_port_write(camera->port, &response, 1) >= 0)
+		if (gp_port_write(camera->port, (char *)&response, 1) >= 0)
 			return (GP_OK);
 	};
 
@@ -184,7 +185,7 @@ static int dc210_read_single_char
 
 	for (i = 0; i < RETRIES; i++){
 
-		error = gp_port_read(camera->port, response, 1);
+		error = gp_port_read(camera->port, (char *)response, 1);
 
 		if (error < 0){
 			if (error == GP_ERROR_TIMEOUT)
@@ -237,7 +238,7 @@ static int dc210_execute_command
 		};
 
 		for (k = 0; k < RETRIES; k++){
-			error = gp_port_read(camera->port, &response, 1);
+			error = gp_port_read(camera->port, (char *)&response, 1);
 			if (error != 1){
 				if (error == GP_ERROR_TIMEOUT){
 					dc210_cmd_error = DC210_TIMEOUT_ERROR;
@@ -306,7 +307,7 @@ static int dc210_write_command_packet
 
 		/* read answer */
 
-		error = gp_port_read(camera->port, &answer, 1);
+		error = gp_port_read(camera->port, (char *)&answer, 1);
 
 		if (error < 0) return GP_ERROR;
 
@@ -387,7 +388,7 @@ static int dc210_read_single_block
 
     error = 1;
     for (k = 0; k < RETRIES; k++){
-      if (gp_port_read(camera->port, b, blocksize) < 0){
+      if (gp_port_read(camera->port, (char *)b, blocksize) < 0){
 	continue;
       };
       error = 0;
@@ -396,7 +397,7 @@ static int dc210_read_single_block
   
     if (error) return GP_ERROR;
 
-    if (dc210_read_single_char(camera, &cs_read) < 0)
+    if (dc210_read_single_char(camera, (unsigned char *)&cs_read) < 0)
       return GP_ERROR;
 
     cs_computed = 0;
@@ -442,7 +443,7 @@ static int dc210_read_to_file
 	  fatal_error = 1;
 	  for (k = 0; k < RETRIES; k++){
 		  /* read packet */
-	          if (gp_port_read(camera->port, b, blocksize) < 0){
+	          if (gp_port_read(camera->port, (char *)b, blocksize) < 0){
 		          dc210_write_single_char(camera, DC210_ILLEGAL_PACKET);
 			  packet_following = dc210_wait_for_response(camera, 0, NULL);
 			  continue;
@@ -463,9 +464,9 @@ static int dc210_read_to_file
 		  };
 		  /* append to file */
 		  if (packets == blocks - 1 && remaining)
-			  gp_file_append(f, b, remaining);
+			  gp_file_append(f, (char *)b, remaining);
 		  else
-			  gp_file_append(f, b, blocksize);
+			  gp_file_append(f, (char *)b, blocksize);
 		  /* request next packet */
 		  dc210_write_single_char(camera, DC210_CORRECT_PACKET);
 		  packet_following = dc210_wait_for_response(camera, 0, NULL);
@@ -624,8 +625,8 @@ static int dc210_format_card (Camera * camera, char * album_name, GPContext * co
   dc210_write_command_packet(camera, data);
   if (dc210_wait_for_response(camera, 3, context) != DC210_PACKET_FOLLOWING) return GP_ERROR;
 
-  gp_port_read(camera->port, answer, 16);
-  gp_port_read(camera->port, &checksum_read, 1);
+  gp_port_read(camera->port, (char *)answer, 16);
+  gp_port_read(camera->port, (char *)&checksum_read, 1);
   checksum = 0;
 
   for (i = 0; i < 16; i++) checksum ^= answer[i];
@@ -657,8 +658,8 @@ static int dc210_get_card_status (Camera * camera, dc210_card_status * card_stat
   dc210_execute_command(camera, cmd);
   if (dc210_wait_for_response(camera, 0, NULL) != DC210_PACKET_FOLLOWING) return GP_ERROR;
 
-  gp_port_read(camera->port, answer, 16);
-  gp_port_read(camera->port, &checksum_read, 1);
+  gp_port_read(camera->port, (char *)answer, 16);
+  gp_port_read(camera->port, (char *)&checksum_read, 1);
 
   checksum = 0;
   for (i = 0; i < 16; i++) checksum ^= answer[i];
@@ -697,7 +698,7 @@ int dc210_format_callback(Camera * camera, CameraWidget * widget, GPContext * co
 
 static int dc210_check_battery (Camera *camera){
 
-  unsigned char cmd[8];
+  char cmd[8];
   dc210_cmd_init(cmd, DC210_CHECK_BATTERY);
 	
   if (dc210_execute_command(camera, cmd) == GP_ERROR) return GP_ERROR;
@@ -746,7 +747,7 @@ int dc210_init_port (Camera *camera){
 	
 	gp_camera_set_port_speed(camera, 9600);
 	gp_port_send_break(camera->port, 300);
-	GP_SYSTEM_SLEEP(300);
+	usleep(300 * 1000);
 
 	/* Excellent. Now our dummy command should work */
 
@@ -783,7 +784,7 @@ int dc210_set_speed (Camera *camera, int speed) {
 	unsigned char cmd[8];
 	GPPortSettings settings;
 
-	dc210_cmd_init(cmd, DC210_SET_SPEED);
+	dc210_cmd_init((char*)cmd, DC210_SET_SPEED);
 	
 	switch (speed) {
 	case 9600:
@@ -800,7 +801,7 @@ int dc210_set_speed (Camera *camera, int speed) {
 		return (GP_ERROR);
 	};
 
-	if (dc210_execute_command(camera, cmd) == GP_ERROR) return GP_ERROR;
+	if (dc210_execute_command(camera, (char*)cmd) == GP_ERROR) return GP_ERROR;
 
 	gp_port_get_settings (camera->port, &settings);
 	settings.serial.speed = speed;
@@ -1109,7 +1110,7 @@ static void dc210_picinfo_from_block (dc210_picture_info * picinfo, unsigned cha
 	picinfo->f_number = data[26];
 	picinfo->battery = data[27];
 	picinfo->exposure_time = data[28] * 0x1000000 + data[29] * 0x10000 + data[30] * 0x100 + data[31];
-	strncpy(picinfo->image_name, &data[32], 12);
+	strncpy(picinfo->image_name, (char *)&data[32], 12);
 	picinfo->image_name[12] = 0;
 	
 }
@@ -1173,7 +1174,7 @@ int dc210_get_status (Camera *camera, dc210_status *status) {
 	dc210_cmd_init(cmd, DC210_GET_STATUS);
 	
 	if (dc210_execute_command(camera, cmd) == GP_ERROR) return GP_ERROR;
-	if (dc210_read_single_block(camera, data, DC210_STATUS_SIZE) == GP_ERROR) return GP_ERROR;
+	if (dc210_read_single_block(camera, (unsigned char *)data, DC210_STATUS_SIZE) == GP_ERROR) return GP_ERROR;
 	if (dc210_wait_for_response(camera, 0, NULL) != DC210_COMMAND_COMPLETE) return GP_ERROR;
 
 #ifdef DEBUG
