@@ -16,7 +16,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <termios.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
@@ -138,12 +137,15 @@ int dc3200_setup(Camera *camera)
 	 *
 	 */
 
-	unsigned char cmd1[5] = {0x01, dc3200_calc_seqnum(camera), 0x01, 0x00, 0x0F};
-	unsigned char cmd2[8] = {0x01, dc3200_calc_seqnum(camera), 0x80, 0x00, 0x01, 0x81, 0x00, 0x03};
+	unsigned char cmd1[5] = {0x01, 0, 0x01, 0x00, 0x0F};
+	unsigned char cmd2[8] = {0x01, 0, 0x80, 0x00, 0x01, 0x81, 0x00, 0x03};
 
 	unsigned char ack[ACK_PACKET_LEN], resp[DEF_PACKET_LEN];
 	int ack_len = ACK_PACKET_LEN, resp_len = DEF_PACKET_LEN;
 
+	cmd1[1] = dc3200_calc_seqnum(camera);
+	cmd2[1] = dc3200_calc_seqnum(camera);
+
 	if(dc3200_send_command(camera, cmd1, sizeof(cmd1), ack, &ack_len) == GP_ERROR)
 		return GP_ERROR;
 	if(dc3200_check_ack(camera, ack, ack_len) == GP_ERROR)
@@ -196,7 +198,6 @@ int dc3200_get_data(Camera *camera, unsigned char **data, unsigned long *data_le
 	unsigned char ack[ACK_PACKET_LEN], resp[DEF_PACKET_LEN];
 	int ack_len = ACK_PACKET_LEN, resp_len = DEF_PACKET_LEN;
 	unsigned long num_left = 0;
-	unsigned long total = 0;
 	int data_start_pos = 0;
 	unsigned char *ptr_data = NULL;
 	unsigned int pid = 0;
@@ -368,7 +369,6 @@ int dc3200_get_data(Camera *camera, unsigned char **data, unsigned long *data_le
 			/* get the total list length from the data header */
 			*data_len = bytes_to_l(resp[34], resp[35], resp[36], resp[37]);
 			num_left = bytes_to_l(resp[12], resp[13], resp[14], resp[15]);
-			total = num_left;
 			data_start_pos = 39;
 			*data_len -= 1;
 
@@ -472,16 +472,17 @@ int dc3200_get_data(Camera *camera, unsigned char **data, unsigned long *data_le
  */
 int dc3200_cancel_get_data(Camera *camera)
 {
-	unsigned char pkt[20] = {0x01, dc3200_calc_seqnum(camera), 0x80, 0x00, 0x20, 0x03, 0x0d, 0xc1, 0x50, 0xc0,
-		0x00, 0x00, 0x00, 0x06, 0x04, 0x01, 0x00, 0x01, (camera->pl->cmd_seqnum >> 8) & 0xff, camera->pl->cmd_seqnum & 0xff};
+	unsigned char pkt[20] = {0x01, 0, 0x80, 0x00, 0x20, 0x03, 0x0d, 0xc1, 0x50, 0xc0,
+		0x00, 0x00, 0x00, 0x06, 0x04, 0x01, 0x00, 0x01, 0, 0};
 	unsigned char ack[ACK_PACKET_LEN], resp[DEF_PACKET_LEN];
 	int ack_len = ACK_PACKET_LEN, resp_len = DEF_PACKET_LEN;
-	struct timeval timeout;
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 1000;
+
+	pkt[1] = dc3200_calc_seqnum(camera);
+	pkt[18] = (camera->pl->cmd_seqnum >> 8) & 0xff;
+	pkt[19] = camera->pl->cmd_seqnum & 0xff;
 
 	/* wait a bit ... */
-	select(0, 0, 0, 0, &timeout);
+	sleep(1);
 
 	/* clear the buffer */
 	dc3200_clear_read_buffer(camera);
@@ -632,7 +633,7 @@ int dc3200_send_packet(Camera *camera, unsigned char *data, int data_len)
 #ifdef DEBUG
 	dump_buffer(buff, buff_len, "s", 16);
 #endif
-	res = gp_port_write(camera->port, buff, data_len + 3);
+	res = gp_port_write(camera->port, (char *)buff, data_len + 3);
 	free(buff);
 	return res;
 }
@@ -664,7 +665,7 @@ int dc3200_recv_packet(Camera *camera, unsigned char *data, int *data_len)
 	 *
 	 */
 
-	res = gp_port_read(camera->port, &buff[num_read], 1);
+	res = gp_port_read(camera->port, (char *)&buff[num_read], 1);
 
 	while(res >= 0 && fails < READ_RETRIES) {
 		if(res == 0) {
@@ -684,7 +685,7 @@ int dc3200_recv_packet(Camera *camera, unsigned char *data, int *data_len)
 				break;
 			}
 		}
-		res = gp_port_read(camera->port, &buff[num_read], 1);
+		res = gp_port_read(camera->port, (char *)&buff[num_read], 1);
 	}
 
 	if(!complete) {
@@ -697,6 +698,7 @@ int dc3200_recv_packet(Camera *camera, unsigned char *data, int *data_len)
 #endif
 
 	if(dc3200_process_packet(camera, buff, &num_read) == GP_ERROR) {
+		free(buff);
 		return GP_ERROR;
 	}
 
@@ -938,7 +940,7 @@ int dc3200_clear_read_buffer(Camera *camera)
 
 	gp_port_set_timeout(camera->port, 0);
 
-	while(gp_port_read(camera->port, &byte, 1) > 0)
+	while(gp_port_read(camera->port, (char *)&byte, 1) > 0)
 		count++;
 
 	if(count > 0)
