@@ -1,6 +1,6 @@
 /* g3.c
  *
- * Copyright  2003 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright 2003 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -56,7 +56,7 @@ g3_channel_read(GPPort *port, int *channel, char **buffer, int *len)
 	unsigned char xbuf[0x800];
 	int tocopy, ret, curlen;
 
-	ret = gp_port_read(port, xbuf, 0x800);
+	ret = gp_port_read(port, (char *)xbuf, 0x800);
 	if (ret < GP_OK) { 
 		gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
 		return ret;
@@ -117,18 +117,23 @@ g3_channel_read_bytes(
 		rest = (rest + 9 + 3) & ~3;
 		if (rest < 0x800) rest = 0x800;
 
-		ret = gp_port_read(port, xbuf, rest);
+		ret = gp_port_read(port, (char *)xbuf, rest);
 		if (ret < GP_OK) {
 			gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
+			gp_context_progress_stop (context, id);
+			free(xbuf);
 			return ret;
 		}
 		if (ret != rest) {
 			gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
+			gp_context_progress_stop (context, id);
+			free(xbuf);
 			return ret;
 		}
 
 		if ((xbuf[2] != 0xff) || (xbuf[3] != 0xff)) {
 			gp_log(GP_LOG_ERROR, "g3", "first bytes do not match.\n");
+			gp_context_progress_stop (context, id);
 			free(xbuf);
 			return GP_ERROR_IO;
 		}
@@ -643,6 +648,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		strcpy(info->file.type,"text/plain");
 
 	cmd = malloc(strlen("-FDAT ")+strlen(folder)+1+strlen(filename)+1);
+	if (!cmd) return GP_ERROR_NO_MEMORY;
 	sprintf(cmd, "-FDAT %s/%s", folder,filename);
 	ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
 	if (ret < GP_OK) goto out;
@@ -683,7 +689,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	}
 out:
 	if (reply) free(reply);
-	if (cmd) free(cmd);
+	free(cmd);
 
 	return (GP_OK);
 
@@ -787,7 +793,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		int n = 0, channel, len, rlen;
 		ret = g3_channel_read(camera->port, &channel, &buf, &len); /* data. */
 		if (ret < GP_OK) goto out;
-		g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
+		ret = g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
 		if (ret < GP_OK) goto out;
 		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 
