@@ -19,9 +19,10 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
+#define _BSD_SOURCE
 
 #include "config.h"
 
@@ -347,7 +348,8 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 
 	Camera *camera = user_data;
 	unsigned char *data = NULL;
-	int size, number, filetype, flash_file_count = 0;
+	int number, filetype, flash_file_count = 0;
+	unsigned int size;
 
 	CHECK (number =
 	       gp_filesystem_number (camera->fs, folder, filename, context));
@@ -408,7 +410,7 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 	}
 	if (!data)
 		return GP_ERROR;
-	return gp_file_set_data_and_size (file, data, size);
+	return gp_file_set_data_and_size (file, (char *)data, size);
 }
 
 static int
