@@ -24,7 +24,7 @@
 
 static void errordumper(GPLogLevel level, const char *domain, const char *str,
                  void *data) {
-  printf("%s\n", str);
+  printf("%s (data %p)\n", str,data);
 }
 
 /* This seems to have no effect on where images go
@@ -87,6 +87,7 @@ void set_capturetarget(Camera *canon, GPContext *canoncontext) {
 }
 */
 
+#if 0
 static void
 capture_to_file(Camera *canon, GPContext *canoncontext, char *fn) {
 	int fd, retval;
@@ -114,6 +115,7 @@ capture_to_file(Camera *canon, GPContext *canoncontext, char *fn) {
 
 	gp_file_free(canonfile);
 }
+#endif
 
 int
 main(int argc, char **argv) {
@@ -121,7 +123,7 @@ main(int argc, char **argv) {
 	int	i, retval;
 	GPContext *canoncontext = sample_create_context();
 
-	gp_log_add_func(GP_LOG_ERROR, errordumper, NULL);
+	gp_log_add_func(GP_LOG_ERROR, errordumper, 0);
 	gp_camera_new(&canon);
 
 	/* When I set GP_LOG_DEBUG instead of GP_LOG_ERROR above, I noticed that the
