@@ -17,8 +17,8 @@
  * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * \note
  * This file contains internal functions. Use of these functions from
@@ -43,9 +43,7 @@
 
 #include <gphoto2/gphoto2-result.h>
 
-#define CHECK_NULL(r)        {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
 #define CHECK_RESULT(result) {int r = (result); if (r < 0) return (r);}
-#define CHECK_MEM(m)         {if (!(m)) return (GP_ERROR_NO_MEMORY);}
 
 /* lengt of one path component */
 #ifndef MAX_PATH
@@ -85,12 +83,9 @@ struct _CameraFile {
 int
 gp_file_new (CameraFile **file)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
-	*file = malloc (sizeof (CameraFile));
-	if (!*file)
-		return (GP_ERROR_NO_MEMORY);
-	memset (*file, 0, sizeof (CameraFile));
+	C_MEM (*file = calloc (1, sizeof (CameraFile)));
 
 	strcpy ((*file)->mime_type, "unknown/unknown");
 	(*file)->ref_count = 1;
@@ -108,12 +103,9 @@ gp_file_new (CameraFile **file)
 int
 gp_file_new_from_fd (CameraFile **file, int fd)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
-	*file = malloc (sizeof (CameraFile));
-	if (!*file)
-		return (GP_ERROR_NO_MEMORY);
-	memset (*file, 0, sizeof (CameraFile));
+	C_MEM (*file = calloc (1, sizeof (CameraFile)));
 
 	strcpy ((*file)->mime_type, "unknown/unknown");
 	(*file)->ref_count = 1;
@@ -132,12 +124,9 @@ gp_file_new_from_fd (CameraFile **file, int fd)
 int
 gp_file_new_from_handler (CameraFile **file, CameraFileHandler* handler, void*private)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
-	*file = malloc (sizeof (CameraFile));
-	if (!*file)
-		return (GP_ERROR_NO_MEMORY);
-	memset (*file, 0, sizeof (CameraFile));
+	C_MEM (*file = calloc (1, sizeof (CameraFile)));
 
 	strcpy ((*file)->mime_type, "unknown/unknown");
 	(*file)->ref_count = 1;
@@ -156,7 +145,7 @@ gp_file_new_from_handler (CameraFile **file, CameraFileHandler* handler, void*pr
  **/
 int gp_file_free (CameraFile *file)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	CHECK_RESULT (gp_file_clean (file));
 	
@@ -176,7 +165,7 @@ int gp_file_free (CameraFile *file)
 int
 gp_file_ref (CameraFile *file)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	file->ref_count += 1;
 	
@@ -193,7 +182,7 @@ gp_file_ref (CameraFile *file)
 int
 gp_file_unref (CameraFile *file)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 	
 	file->ref_count -= 1;
 
@@ -215,33 +204,24 @@ int
 gp_file_append (CameraFile *file, const char *data, 
 		unsigned long int size)
 {
-        char *t;
-
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
-		if (!file->data)
-			file->data = malloc (sizeof(char) * (size));
-		else {
-			t = realloc (file->data, sizeof (char) * (file->size + size));
-			if (!t)
-				return GP_ERROR_NO_MEMORY;
-			file->data = (unsigned char*)t;
-		}
+		C_MEM (file->data = realloc (file->data, sizeof (char) * (file->size + size)));
 		memcpy (&file->data[file->size], data, size);
 		file->size += size;
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
 		unsigned long int curwritten = 0; 
 		while (curwritten < size) {
-			size_t	res = write (file->fd, data+curwritten, size-curwritten);
+			ssize_t	res = write (file->fd, data+curwritten, size-curwritten);
 			if (res == -1) {
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d writing to fd.", errno);
+				GP_LOG_E ("Encountered error %d writing to fd.", errno);
 				return GP_ERROR_IO_WRITE;
 			}
 			if (!res) { /* no progress is bad too */
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered 0 bytes written to fd.");
+				GP_LOG_E ("Encountered 0 bytes written to fd.");
 				return GP_ERROR_IO_WRITE;
 			}
 			curwritten += res;
@@ -251,14 +231,11 @@ gp_file_append (CameraFile *file, const char *data,
 	case GP_FILE_ACCESSTYPE_HANDLER: {
 		uint64_t	xsize = size;
 		/* FIXME: assume we write one blob */
-		if (!file->handler->write) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "write handler is NULL");
-			return GP_ERROR_BAD_PARAMETERS;
-		}
+		C_PARAMS (file->handler->write);
 		return file->handler->write (file->private, (unsigned char*)data, &xsize);
 	}
 	default:
-		gp_log (GP_LOG_ERROR, "gphoto2-file", "Unknown file access type %d", file->accesstype);
+		GP_LOG_E ("Unknown file access type %d", file->accesstype);
 		return GP_ERROR;
 	}
         return (GP_OK);
@@ -276,7 +253,7 @@ int
 gp_file_slurp (CameraFile *file, char *data, 
 	size_t size, size_t *readlen
 ) {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
@@ -289,13 +266,13 @@ gp_file_slurp (CameraFile *file, char *data,
 	case GP_FILE_ACCESSTYPE_FD: {
 		unsigned long int curread = 0; 
 		while (curread < size) {
-			size_t	res = read (file->fd, data+curread, size-curread);
+			ssize_t	res = read (file->fd, data+curread, size-curread);
 			if (res == -1) {
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d reading from fd.", errno);
+				GP_LOG_E ("Encountered error %d reading from fd.", errno);
 				return GP_ERROR_IO_READ;
 			}
 			if (!res) { /* no progress is bad too */
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered 0 bytes reading from fd.");
+				GP_LOG_E ("Encountered 0 bytes reading from fd.");
 				return GP_ERROR_IO_READ;
 			}
 			curread += res;
@@ -308,18 +285,15 @@ gp_file_slurp (CameraFile *file, char *data,
 		uint64_t	xsize = size;
 		int		ret;
 
-		if (!file->handler->read) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "read handler is NULL");
-			return GP_ERROR_BAD_PARAMETERS;
-		}
+		C_PARAMS (file->handler->read);
 		ret = file->handler->read (file->private, (unsigned char*)data, &xsize);
 		*readlen = xsize;
 		if (ret != GP_OK)
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "File handler read returned %d", ret);
+			GP_LOG_E ("File handler read returned %d", ret);
 		return ret;
 	}
 	default:
-		gp_log (GP_LOG_ERROR, "gphoto2-file", "Unknown file access type %d", file->accesstype);
+		GP_LOG_E ("Unknown file access type %d", file->accesstype);
 		return GP_ERROR;
 	}
         return (GP_OK);
@@ -338,35 +312,34 @@ int
 gp_file_set_data_and_size (CameraFile *file, char *data,
 			   unsigned long int size)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
-		if (file->data)
-			free (file->data);
+		free (file->data);
 		file->data = (unsigned char*)data;
 		file->size = size;
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
-		int curwritten = 0;
+		unsigned int curwritten = 0;
 
 		/* truncate */
 		if (-1 == lseek (file->fd, 0, SEEK_SET)) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseeking to 0.", errno);
+			GP_LOG_E ("Encountered error %d lseeking to 0.", errno);
 			/* might happen on pipes ... just ignore it */
 		}
 		if (-1 == ftruncate (file->fd, 0)) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d ftruncating to 0.", errno);
+			GP_LOG_E ("Encountered error %d ftruncating to 0.", errno);
 			/* might happen on pipes ... just ignore it */
 		}
 		while (curwritten < size) {
-			size_t	res = write (file->fd, data+curwritten, size-curwritten);
+			ssize_t	res = write (file->fd, data+curwritten, size-curwritten);
 			if (res == -1) {
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d writing to fd.", errno);
+				GP_LOG_E ("Encountered error %d writing to fd.", errno);
 				return GP_ERROR_IO_WRITE;
 			}
 			if (!res) { /* no progress is bad too */
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered 0 bytes written to fd.");
+				GP_LOG_E ("Encountered 0 bytes written to fd.");
 				return GP_ERROR_IO_WRITE;
 			}
 			curwritten += res;
@@ -381,14 +354,11 @@ gp_file_set_data_and_size (CameraFile *file, char *data,
 		uint64_t	xsize = size;
 		int		ret;
 
-		if (!file->handler->write) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "write handler is NULL");
-			return GP_ERROR_BAD_PARAMETERS;
-		}
+		C_PARAMS (file->handler->write);
 		/* FIXME: handle multiple blob writes */
 		ret = file->handler->write (file->private, (unsigned char*)data, &xsize);
 		if (ret != GP_OK) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Handler data() returned %d", ret);
+			GP_LOG_E ("Handler data() returned %d", ret);
 			return ret;
 		}
 		/* This function takes over the responsibility for "data", aka
@@ -398,7 +368,7 @@ gp_file_set_data_and_size (CameraFile *file, char *data,
 		return GP_OK;
 	}
 	default:
-		gp_log (GP_LOG_ERROR, "gphoto2-file", "Unknown file access type %d", file->accesstype);
+		GP_LOG_E ("Unknown file access type %d", file->accesstype);
 		return GP_ERROR;
 	}
 	return (GP_OK);
@@ -423,7 +393,7 @@ int
 gp_file_get_data_and_size (CameraFile *file, const char **data,
 			   unsigned long int *size)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
@@ -443,30 +413,28 @@ gp_file_get_data_and_size (CameraFile *file, const char **data,
 		}
 		if (-1 == (offset = lseek (file->fd, 0, SEEK_CUR))) {
 			/* should not happen if we passed the above case */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to CUR.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to CUR.", errno);
 			return GP_ERROR_IO_READ;
 		}
 		if (-1 == lseek (file->fd, 0, SEEK_SET)) {
 			/* should not happen if we passed the above cases */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to CUR.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to CUR.", errno);
 			return GP_ERROR_IO_READ;
 		}
 		if (size) *size = offset;
 		if (!data) /* just the size... */
 			return GP_OK;
-		*data = malloc (offset);
-		if (!*data)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (*data = malloc (offset));
 		while (curread < offset) {
-			unsigned int res = read (file->fd, (char*)((*data)+curread), offset-curread);
+			ssize_t res = read (file->fd, (char*)((*data)+curread), offset-curread);
 			if (res == -1) {
 				free ((char*)*data);
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d reading.", errno);
+				GP_LOG_E ("Encountered error %d reading.", errno);
 				return GP_ERROR_IO_READ;
 			}
 			if (res == 0) {
 				free ((char*)*data);
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "No progress during reading.");
+				GP_LOG_E ("No progress during reading.");
 				return GP_ERROR_IO_READ;
 			}
 			curread += res;
@@ -477,31 +445,26 @@ gp_file_get_data_and_size (CameraFile *file, const char **data,
 		uint64_t	xsize = 0;
 		int		ret;
 
-		if (!file->handler->read) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "read handler is NULL");
-			return GP_ERROR_BAD_PARAMETERS;
-		}
+		C_PARAMS (file->handler->read);
 		ret = file->handler->size (file->private, &xsize);
 		if (ret != GP_OK) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d querying size().", ret);
+			GP_LOG_E ("Encountered error %d querying size().", ret);
 			return ret;
 		}
 		if (size) *size = xsize;
 		if (!data) /* just the size... */
 			return GP_OK;
-		*data = malloc (xsize);
-		if (!*data)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (*data = malloc (xsize));
 		ret = file->handler->read (file->private, (unsigned char*)*data, &xsize);
 		if (ret != GP_OK) {
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d getting data().", ret);
+			GP_LOG_E ("Encountered error %d getting data().", ret);
 			free ((char*)*data);
 			*data = NULL;
 		}
 		return ret;
 	}
 	default:
-		gp_log (GP_LOG_ERROR, "gphoto2-file", "Unknown file access type %d", file->accesstype);
+		GP_LOG_E ("Unknown file access type %d", file->accesstype);
 		return GP_ERROR;
 	}
 	return (GP_OK);
@@ -520,16 +483,14 @@ gp_file_save (CameraFile *file, const char *filename)
 	FILE *fp;
 	struct utimbuf u;
 
-	CHECK_NULL (file && filename);
+	C_PARAMS (file && filename);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
 		if (!(fp = fopen (filename, "wb")))
 			return GP_ERROR;
 		if (fwrite (file->data, (size_t)sizeof(char), (size_t)file->size, fp) != (size_t)file->size) {
-			gp_log (GP_LOG_ERROR, "libgphoto2",
-				"Not enough space on device in "
-				"order to save '%s'.", filename);
+			GP_LOG_E ("Not enough space on device in order to save '%s'.", filename);
 			fclose (fp);
 			unlink (filename);
 			return GP_ERROR;
@@ -545,19 +506,19 @@ gp_file_save (CameraFile *file, const char *filename)
 			return GP_ERROR_IO;
 		if (-1 == (offset = lseek (file->fd, 0, SEEK_CUR))) {
 			/* should not happen if we passed the above case */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to CUR.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to CUR.", errno);
 			return GP_ERROR_IO_READ;
 		}
 		if (-1 == lseek (file->fd, 0, SEEK_SET)) {
 			/* should not happen if we passed the above case */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to BEGIN.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to BEGIN.", errno);
 			return GP_ERROR_IO_READ;
 		}
-		data = malloc(65536);
-		if (!data)
-			return GP_ERROR_NO_MEMORY;
-		if (!(fp = fopen (filename, "wb")))
+		C_MEM (data = malloc(65536));
+		if (!(fp = fopen (filename, "wb"))) {
+			free (data);
 			return GP_ERROR;
+		}
 		while (curread < offset) {
 			int toread, res;
 
@@ -572,9 +533,7 @@ gp_file_save (CameraFile *file, const char *filename)
 				return GP_ERROR_IO_READ;
 			}
 			if (fwrite (data, 1, res, fp) != res) {
-				gp_log (GP_LOG_ERROR, "libgphoto2",
-					"Not enough space on device in "
-					"order to save '%s'.", filename);
+				GP_LOG_E ("Not enough space on device in order to save '%s'.", filename);
 				free (data);
 				fclose (fp);
 				unlink (filename);
@@ -587,7 +546,7 @@ gp_file_save (CameraFile *file, const char *filename)
 		break;
 	}
 	default:
-		gp_log (GP_LOG_ERROR, "gphoto2-file", "Unknown file access type %d", file->accesstype);
+		GP_LOG_E ("Unknown file access type %d", file->accesstype);
 		return GP_ERROR;
 	}
 
@@ -626,6 +585,7 @@ static const char *mime_table[] = {
     "raw",  GP_MIME_RAW,
     "mts",  GP_MIME_AVCHD,
     "m2ts", GP_MIME_AVCHD,
+    "arw",  GP_MIME_ARW,
     NULL};
 /**
  * @param file a #CameraFile
@@ -643,7 +603,7 @@ gp_file_open (CameraFile *file, const char *filename)
 	struct stat s;
 
 
-	CHECK_NULL (file && filename);
+	C_PARAMS (file && filename);
 
 	CHECK_RESULT (gp_file_clean (file));
 
@@ -657,8 +617,10 @@ gp_file_open (CameraFile *file, const char *filename)
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
 		file->data = malloc (sizeof(char)*(size + 1));
-		if (!file->data)
+		if (!file->data) {
+			fclose (fp);
 			return (GP_ERROR_NO_MEMORY);
+		}
 		size_read = fread (file->data, (size_t)sizeof(char), (size_t)size, fp);
 		if (ferror(fp)) {
 			gp_file_clean (file);
@@ -675,7 +637,7 @@ gp_file_open (CameraFile *file, const char *filename)
 			fclose (fp);
 			break;
 		}
-		gp_log (GP_LOG_ERROR, "gp_file_open", "Needs to be initialized with fd=-1 to work");
+		GP_LOG_E ("Needs to be initialized with fd=-1 to work");
 		fclose (fp);
 		return GP_ERROR;
 	}
@@ -735,12 +697,11 @@ gp_file_clean (CameraFile *file)
 	 * This is used to prep a CameraFile struct to be filled.
          */
 
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
-		if (file->data != NULL)
-			free(file->data);
+		free (file->data);
 		file->data = NULL;
 		file->size = 0;
 		break;
@@ -761,10 +722,9 @@ gp_file_clean (CameraFile *file)
 int
 gp_file_copy (CameraFile *destination, CameraFile *source)
 {
-	CHECK_NULL (destination && source);
+	C_PARAMS (destination && source);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-file", "Copying '%s' onto '%s'...",
-		source->name, destination->name);
+	GP_LOG_D ("Copying '%s' onto '%s'...", source->name, destination->name);
 
 	/* struct members we can just copy. All generic ones, but not refcount. */
 	memcpy (destination->name, source->name, sizeof (source->name));
@@ -773,14 +733,10 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 
 	if ((destination->accesstype == GP_FILE_ACCESSTYPE_MEMORY) &&
 	    (source->accesstype == GP_FILE_ACCESSTYPE_MEMORY)) {
-		if (destination->data) {
-			free (destination->data);
-			destination->data = NULL;
-		}
+		free (destination->data);
+		destination->data = NULL;
 		destination->size = source->size;
-		destination->data = malloc (sizeof (char) * source->size);
-		if (!destination->data)
-			return (GP_ERROR_NO_MEMORY);
+		C_MEM (destination->data = malloc (sizeof (char) * source->size));
 		memcpy (destination->data, source->data, source->size);
 		return (GP_OK);
 	}
@@ -790,10 +746,9 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		off_t	offset;
 		unsigned long int curread = 0;
 
-		if (destination->data) {
-			free (destination->data);
-			destination->data = NULL;
-		}
+		free (destination->data);
+		destination->data = NULL;
+
 		if (-1 == lseek (source->fd, 0, SEEK_END)) {
 			if (errno == EBADF) return GP_ERROR_IO;
 			/* Might happen for pipes or sockets. Umm. Hard. */
@@ -801,28 +756,26 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		}
 		if (-1 == (offset = lseek (source->fd, 0, SEEK_CUR))) {
 			/* should not happen if we passed the above case */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to CUR.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to CUR.", errno);
 			return GP_ERROR_IO_READ;
 		}
 		if (-1 == lseek (source->fd, 0, SEEK_SET)) {
 			/* should not happen if we passed the above cases */
-			gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d lseekin to CUR.", errno);
+			GP_LOG_E ("Encountered error %d lseekin to CUR.", errno);
 			return GP_ERROR_IO_READ;
 		}
 		destination->size = offset;
-		destination->data = malloc (offset);
-		if (!destination->data)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (destination->data = malloc (offset));
 		while (curread < offset) {
-			unsigned int res = read (source->fd, destination->data+curread, offset-curread);
+			ssize_t res = read (source->fd, destination->data+curread, offset-curread);
 			if (res == -1) {
 				free (destination->data);
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "Encountered error %d reading.", errno);
+				GP_LOG_E ("Encountered error %d reading.", errno);
 				return GP_ERROR_IO_READ;
 			}
 			if (res == 0) {
 				free (destination->data);
-				gp_log (GP_LOG_ERROR, "gphoto2-file", "No progress during reading.");
+				GP_LOG_E ("No progress during reading.");
 				return GP_ERROR_IO_READ;
 			}
 			curread += res;
@@ -838,10 +791,10 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		if (-1 == ftruncate (destination->fd, 0))
 			perror("ftruncate");
 		lseek (source->fd, 0, SEEK_SET);
-		data = malloc (65536);
+		C_MEM (data = malloc (65536));
 		while (1) {
 			unsigned long curwritten = 0;
-			int res;
+			ssize_t res;
 
 			res = read (source->fd, data, 65536);
 			if (res == -1) {
@@ -851,7 +804,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 			if (res == 0)
 				break;
 			while (curwritten < res) {
-				int res2 = write (destination->fd, data+curwritten, res-curwritten);
+				ssize_t res2 = write (destination->fd, data+curwritten, res-curwritten);
 				if (res2 == -1) {
 					free (data);
 					return GP_ERROR_IO_WRITE;
@@ -863,6 +816,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 			if (res < 65536) /* end of file */
 				break;
 		}
+		free (data);
 		return GP_OK;
 	}
 	if (	(destination->accesstype == GP_FILE_ACCESSTYPE_FD) &&
@@ -899,7 +853,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		}
 		return GP_OK;
 	}
-	gp_log (GP_LOG_ERROR, "gphoto2-file", "Unhandled cases in gp_copy_file. Bad!");
+	GP_LOG_E ("Unhandled cases in gp_copy_file. Bad!");
 	return GP_ERROR;
 }
 
@@ -912,7 +866,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 int
 gp_file_get_name (CameraFile *file, const char **name)
 {
-	CHECK_NULL (file && name);
+	C_PARAMS (file && name);
 
 	*name = file->name;
 
@@ -942,14 +896,12 @@ gp_file_get_name_by_type (CameraFile *file, const char *basename, CameraFileType
 	const char *suffix = NULL;
 	int i;
 
-	CHECK_NULL (file && basename && newname);
+	C_PARAMS (file && basename && newname);
 	*newname = NULL;
 
 	/* the easy case, always map 1:1, if it has a suffix already. */
 	if ((type == GP_FILE_TYPE_NORMAL) && strchr(basename,'.')) {
-		*newname = strdup (basename);
-		if (!*newname)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (*newname = strdup (basename));
 		return GP_OK;
 	}
 
@@ -972,9 +924,7 @@ gp_file_get_name_by_type (CameraFile *file, const char *basename, CameraFileType
 		int xlen;
 		if (!suffix)
 			suffix = s+1;
-		new = malloc (strlen(prefix) + (s-basename+1) + strlen (suffix) + 1);
-		if (!new)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (new = malloc (strlen(prefix) + (s-basename+1) + strlen (suffix) + 1));
 		strcpy (new, prefix);
 		xlen = strlen (new);
 		memcpy (new+xlen, basename, s-basename+1);
@@ -982,9 +932,7 @@ gp_file_get_name_by_type (CameraFile *file, const char *basename, CameraFileType
 		strcat (new, suffix);
 	} else { /* no dot in basename? */
 		if (!suffix) suffix = "";
-		new = malloc (strlen(prefix) + strlen(basename) + 1 + strlen (suffix) + 1);
-		if (!new)
-			return GP_ERROR_NO_MEMORY;
+		C_MEM (new = malloc (strlen(prefix) + strlen(basename) + 1 + strlen (suffix) + 1));
 		strcpy (new, prefix);
 		strcat (new, basename);
 		if (strlen(suffix)) {
@@ -1006,7 +954,7 @@ gp_file_get_name_by_type (CameraFile *file, const char *basename, CameraFileType
 int
 gp_file_get_mime_type (CameraFile *file, const char **mime_type)
 {
-	CHECK_NULL (file && mime_type);
+	C_PARAMS (file && mime_type);
 
 	*mime_type = file->mime_type;
 
@@ -1023,7 +971,7 @@ gp_file_get_mime_type (CameraFile *file, const char **mime_type)
 int
 gp_file_set_name (CameraFile *file, const char *name)
 {
-	CHECK_NULL (file && name);
+	C_PARAMS (file && name);
 
 	strncpy (file->name, name, sizeof (file->name));
 
@@ -1040,7 +988,7 @@ gp_file_set_name (CameraFile *file, const char *name)
 int
 gp_file_set_mime_type (CameraFile *file, const char *mime_type)
 {
-	CHECK_NULL (file && mime_type);
+	C_PARAMS (file && mime_type);
 
 	strncpy (file->mime_type, mime_type, sizeof (file->mime_type));
 
@@ -1060,7 +1008,7 @@ gp_file_detect_mime_type (CameraFile *file)
 				        (char) 0x00, (char) 0x08, '\0' };
 	const char JPEG_SOI_MARKER[] = {(char) 0xFF, (char) 0xD8, '\0' };
 
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	switch (file->accesstype) {
 	case GP_FILE_ACCESSTYPE_MEMORY:
@@ -1127,10 +1075,9 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 		GP_MIME_CR2,  "cr2",
 		NULL};
 
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-file", "Adjusting file name for "
-		"mime type '%s'...", file->mime_type);
+	GP_LOG_D ("Adjusting file name for mime type '%s'...", file->mime_type);
 	for (x = 0; table[x]; x += 2)
                 if (!strcmp (file->mime_type, table[x])) {
 
@@ -1141,8 +1088,7 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 			strcat (file->name, table[x + 1]);
 			break;
 		}
-	gp_log (GP_LOG_DEBUG, "gphoto2-file", "Name adjusted to '%s'.",
-		file->name);
+	GP_LOG_D ("Name adjusted to '%s'.", file->name);
 	
 	return (GP_OK);
 }
@@ -1157,7 +1103,7 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 int
 gp_file_get_mtime (CameraFile *file, time_t *mtime)
 {
-	CHECK_NULL (file && mtime);
+	C_PARAMS (file && mtime);
 
 	*mtime = file->mtime;
 
@@ -1174,7 +1120,7 @@ gp_file_get_mtime (CameraFile *file, time_t *mtime)
 int
 gp_file_set_mtime (CameraFile *file, time_t mtime)
 {
-	CHECK_NULL (file);
+	C_PARAMS (file);
 
 	file->mtime = mtime;
 
