@@ -1,6 +1,6 @@
 /** \file
  *
- * \author Copyright 2001 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 2001 Lutz Mueller <lutz@users.sf.net>
  *
  * \par License
  * This library is free software; you can redistribute it and/or
@@ -17,10 +17,12 @@
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 #include <gphoto2/gphoto2-port-log.h>
 
@@ -79,20 +81,10 @@ static unsigned int log_funcs_count = 0;
 int
 gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
 {
-	LogFunc *new_log_funcs;
+	C_PARAMS (func);
 
-	if (!func)
-		return (GP_ERROR_BAD_PARAMETERS);
-
-	if (!log_funcs)
-		new_log_funcs = malloc (sizeof (LogFunc));
-	else
-		new_log_funcs = realloc (log_funcs, sizeof (LogFunc) * 
-					 (log_funcs_count + 1));
-	if (!new_log_funcs)
-		return (GP_ERROR_NO_MEMORY);
-
-	log_funcs = new_log_funcs;
+	C_MEM (log_funcs = realloc (log_funcs, sizeof (LogFunc) *
+				(log_funcs_count + 1)));
 	log_funcs_count++;
 
 	log_funcs[log_funcs_count - 1].id = log_funcs_count;
@@ -114,8 +106,7 @@ gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
 int
 gp_log_remove_func (int id)
 {
-	if (id < 1 || id > log_funcs_count)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (id > 0 && id <= log_funcs_count);
 
 	memmove (log_funcs + id - 1, log_funcs + id, log_funcs_count - id);
 	log_funcs_count--;
@@ -174,6 +165,7 @@ gp_log_remove_func (int id)
  * Takes the data and creates a formatted hexdump string. If you would like
  * to log text messages, use #gp_log instead.
  **/
+/* coverity[-tainted_sink] */
 void
 gp_log_data (const char *domain, const char *data, unsigned int size)
 {
@@ -181,29 +173,28 @@ gp_log_data (const char *domain, const char *data, unsigned int size)
 	char *curline, *result;
 	int x = HEXDUMP_INIT_X;
 	int y = HEXDUMP_INIT_Y;
-	int index;
+	unsigned int index;
 	unsigned char value;
 
 	if (!data) {
-		gp_log (GP_LOG_DATA, domain, _("No hexdump (NULL buffer)"));
+		gp_log (GP_LOG_DATA, domain, "No hexdump (NULL buffer)");
 		return;
 	}
 
 	if (!size) {
-		gp_log (GP_LOG_DATA, domain, _("Empty hexdump of empty buffer"));
+		gp_log (GP_LOG_DATA, domain, "Empty hexdump of empty buffer");
 		return;
 	}
 
 	if (size > 1024*1024) {
 		/* Does not make sense for 200 MB movies */
-		gp_log (GP_LOG_DATA, domain, _("Truncating dump from %d bytes to 1MB"), size);
+		gp_log (GP_LOG_DATA, domain, "Truncating dump from %d bytes to 1MB", size);
 		size = 1024*1024;
 	}
 
 	curline = result = malloc ((HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
 	if (!result) {
-		gp_log (GP_LOG_ERROR, "gphoto2-log", _("Malloc for %i bytes "
-			"failed"), (HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
+		GP_LOG_E ("Malloc for %i bytes failed", (HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
 		return;
 	}
 
@@ -233,7 +224,7 @@ gp_log_data (const char *domain, const char *data, unsigned int size)
         }
         curline[0] = '\0';
 
-	gp_log (GP_LOG_DATA, domain, _("Hexdump of %i = 0x%x bytes follows:\n%s"),
+	gp_log (GP_LOG_DATA, domain, "Hexdump of %i = 0x%x bytes follows:\n%s",
 		size, size, result);
 	free (result);
 }
@@ -260,7 +251,7 @@ void
 gp_logv (GPLogLevel level, const char *domain, const char *format,
 	 va_list args)
 {
-	int i;
+	unsigned int i;
 #ifdef HAVE_VA_COPY
 	va_list xargs;
 #else
@@ -325,6 +316,21 @@ gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 	va_end (args);
 }
 
+void
+gp_log_with_source_location(GPLogLevel level, const char *file, int line, const char *func, const char *format, ...)
+{
+	va_list args;
+        char domain[100];
+
+        /* Only display filename without any path/directory part */
+        file = strrchr(file, '/') ? strrchr(file, '/') + 1 : file;
+        snprintf(domain, sizeof(domain), "%s [%s:%d]", func, file, line);
+
+	va_start (args, format);
+	gp_logv (level, domain, format, args);
+	va_end (args);
+}
+
 #else /* DISABLE_DEBUGGING */
 
 /*
@@ -348,6 +354,9 @@ gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 #ifdef gp_log
 #undef gp_log
 #endif
+#ifdef gp_log_with_source_location
+#undef gp_log_with_source_location
+#endif
 
 int
 gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
@@ -376,6 +385,11 @@ void
 gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 {
 }
+
+void
+gp_log_with_source_location(GPLogLevel level, const char *file, int line, const char *func, const char *format, ...)
+{
+}
 #endif /* DISABLE_DEBUGGING */
 
 
