@@ -1,21 +1,5 @@
 var NAVTREEINDEX5 =
 {
-"struct__CameraFileInfoFile.html#ae7acce7dab662a7dfb088602176ac20b":[1,0,6,2],
-"struct__CameraFileInfoFile.html#af076fe6da9612a0167acf489c40f31e8":[1,0,6,1],
-"struct__CameraFileInfoPreview.html":[1,0,7],
-"struct__CameraFileInfoPreview.html#a08eb4d953d2c9f4f7e8ce54223bb8ea9":[1,0,7,3],
-"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a":[1,0,7,5],
-"struct__CameraFileInfoPreview.html#a2cc5f80f66aa90d7c276df9ecd530490":[1,0,7,1],
-"struct__CameraFileInfoPreview.html#a42e378c8d0668052b39c0cca22382dd7":[1,0,7,4],
-"struct__CameraFileInfoPreview.html#a872f4d1625b0008c625138f33ef3e4cb":[1,0,7,0],
-"struct__CameraFileInfoPreview.html#a9a3b70779dc8cea4d520b9349bee6ba4":[1,0,7,2],
-"struct__CameraFilesystem.html":[1,0,8],
-"struct__CameraFilesystem.html#a1df04156f0ffe145f2e9d99eea9641ee":[1,0,8,13],
-"struct__CameraFilesystem.html#a2f676eede09ea213a0ef7e8711d6ddf3":[1,0,8,5],
-"struct__CameraFilesystem.html#a31e9fa7b813074658672a512b683a334":[1,0,8,1],
-"struct__CameraFilesystem.html#a32c4b277105e1a86ade9700dbc1cf891":[1,0,8,4],
-"struct__CameraFilesystem.html#a38fcb7940178fa609cd62b8aa9ea5c29":[1,0,8,10],
-"struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd":[1,0,8,12],
 "struct__CameraFilesystem.html#a4f95116fa657186a1cb011004451d503":[1,0,8,6],
 "struct__CameraFilesystem.html#a6a7b0038371ba041f35153ad55e302ba":[1,0,8,16],
 "struct__CameraFilesystem.html#a93e23f96c30c239d9db008156eb9ecf1":[1,0,8,9],
@@ -249,5 +233,21 @@ var NAVTREEINDEX5 =
 "template_8c.html#a60705b0c4e51ef116aeb4a36307ab2ba":[2,0,0,0,0,5],
 "template_8c.html#a6b9e41589d48154298976ee65aa9f9d6":[2,0,0,0,0,9],
 "template_8c.html#a6fb50401c2b5419a38d7cdec8468b6ee":[2,0,0,0,0,18],
-"template_8c.html#a75278405e7f034d2b1af80bfd94675fe":[2,0,0,0,0,1]
+"template_8c.html#a75278405e7f034d2b1af80bfd94675fe":[2,0,0,0,0,1],
+"template_8c.html#a8f4cb0c9cb393986b466b60edc3bcd98":[2,0,0,0,0,19],
+"template_8c.html#a9127c82b044f56fac4cbceb92b9e0217":[2,0,0,0,0,22],
+"template_8c.html#aa2bedbf50a33963dbe0938bc83d542d1":[2,0,0,0,0,11],
+"template_8c.html#aab9c38421fc9b44444b9586c3e117d91":[2,0,0,0,0,8],
+"template_8c.html#ab19683ddf354e8f4d401d79489cdab00":[2,0,0,0,0,13],
+"template_8c.html#abaedcea6348683a300838b493f4254a9":[2,0,0,0,0,17],
+"template_8c.html#abc05a8ae36f010d3b6d1f774a397d9b5":[2,0,0,0,0,15],
+"template_8c.html#ac5604f4c487f848626ed2ac56a7c9339":[2,0,0,0,0,20],
+"template_8c.html#ad6759410a0799b25df88f4297d15441f":[2,0,0,0,0,14],
+"template_8c.html#addfb85248f27b40480209cc9de04c6a0":[2,0,0,0,0,3],
+"template_8c.html#af8be80f9a636a9dff8c497cf3cfe0871":[2,0,0,0,0,10],
+"union__GPPortSettings.html":[1,0,24],
+"union__GPPortSettings.html#a0aefcc2253c254cac9ea3fb15f2a059b":[1,0,24,1],
+"union__GPPortSettings.html#a5e89705555ef3dd1abc7baca312cd253":[1,0,24,3],
+"union__GPPortSettings.html#a9fa1454110af3b6425425de105284c8e":[1,0,24,2],
+"union__GPPortSettings.html#ac5d97643d4d2c83edd17731949966b49":[1,0,24,0]
 };
