@@ -1,8 +1,8 @@
 /* print-camera-list - print libgphoto2 camera list in different formats
  *
- * Copyright  2002,2005 Hans Ulrich Niedermann <hun@users.sourceforge.net>
- * Portions Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
- * Portions Copyright  2005 Julien BLACHE <jblache@debian.org>
+ * Copyright 2002,2005 Hans Ulrich Niedermann <hun@users.sourceforge.net>
+ * Portions Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Portions Copyright 2005 Julien BLACHE <jblache@debian.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 
@@ -70,8 +70,6 @@ typedef struct {
 	GPVersionFunc version_func;
 } module_version;
 
-
-
 const module_version module_versions[] = {
 	{ "libgphoto2", gp_library_version },
 	{ "libgphoto2_port", gp_port_library_version },
@@ -92,6 +90,8 @@ typedef struct {
 
 typedef int (* begin_func_t)  (const func_params_t *params,
 			       void **data);
+typedef int (* middle_func_t)  (const func_params_t *params,
+			       void **data);
 typedef int (* camera_func_t) (const func_params_t *params, 
 			       const int i,
 			       const int total,
@@ -165,9 +165,9 @@ print_version_comment(FILE *out,
 	unsigned int n;
 	if (out == NULL) { FATAL("Internal error: NULL out in print_version_comment()"); }
 	if (firstline != NULL) { fputs(firstline, out); }
-	fputs(startline, out);
+	if (startline != NULL) { fputs(startline, out); }
 	fputs("Created from this library:", out);
-	fputs(endline, out);
+	if (endline != NULL) { fputs(endline, out); }
 	for (n=0; (module_versions[n].name != NULL) && (module_versions[n].version_func != NULL); n++) {
 		const char *name = module_versions[n].name;
 		GPVersionFunc func = module_versions[n].version_func;
@@ -220,7 +220,6 @@ hotplug_camera_func (const func_params_t *params,
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
-	int usb_vendor = 0, usb_product = 0;
 	const char *usermap_script = 
 		((*params->argv)[0] != NULL)
 		?((*params->argv)[0])
@@ -233,8 +232,6 @@ hotplug_camera_func (const func_params_t *params,
 			proto = 0;
 			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
 				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-			usb_vendor = a->usb_vendor;
-			usb_product = a->usb_product;
 		} else if ((a->usb_class) && (a->usb_class != 666)) {
 			class = a->usb_class;
 			subclass = a->usb_subclass;
@@ -248,8 +245,6 @@ hotplug_camera_func (const func_params_t *params,
 				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
 			else
 				proto = 0;
-			usb_vendor = 0;
-			usb_product = 0;
 		}
 	} else {
 		/* not a USB camera */
@@ -317,7 +312,7 @@ human_end_func (const func_params_t *params, void *data)
 
 /** C equivalent of basename(1) */
 static const char *
-basename (const char *pathname)
+path_basename (const char *pathname)
 {
 	char *result, *tmp;
 	/* remove path part from camlib name */
@@ -339,7 +334,7 @@ human_camera_func (const func_params_t *params,
 		   void *data)
 {
 	const char *camlib_basename;
-	camlib_basename = basename(a->library);
+	camlib_basename = path_basename(a->library);
 	printf("%3d|%-20s|%-20s|%s\n",
 	       i+1, 
 	       camlib_basename,
@@ -368,12 +363,18 @@ idlist_camera_func (const func_params_t *params,
 
 typedef enum {
 		UDEV_PRE_0_98 = 0,
-		UDEV_0_98 = 1
+		UDEV_0_98 = 1,
+		UDEV_136 = 2,
+		UDEV_175 = 3,
+		UDEV_201 = 4
 } udev_version_t;
 
 static const StringFlagItem udev_version_t_map[] = {
 	{ "pre-0.98", UDEV_PRE_0_98 },
 	{ "0.98", UDEV_0_98 },
+	{ "136", UDEV_136 },
+	{ "175", UDEV_175 },
+	{ "201", UDEV_201 },
 	{ NULL, 0 }
 };
 
@@ -386,6 +387,7 @@ typedef struct {
 	char *script;
 	const char *begin_string;
 	const char *usbcam_string;
+	const char *usbdisk_string;
 } udev_persistent_data_t;
 
 
@@ -397,17 +399,63 @@ udev_parse_params (const func_params_t *params, void **data)
 	 */
 	static const char * const begin_strings[] = {
 		/* UDEV_PRE_0_98 */
-		"BUS!=\"usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n",
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"BUS!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
 		/* UDEV_0_98 */
-		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n"
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
+		/* UDEV_136 */
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
+		"ENV{DEVTYPE}!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n"
+		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{program}=\"usb_id --export %%p\"\n"
+		/* ignore mass storage class having devices in mark-up */
+		"ENV{ID_USB_INTERFACES}==\"*:08*:*\", GOTO=\"libgphoto2_usb_end\"\n"
+		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
+		 * more rules . It will be completed in udev_begin_func() */
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
+		/* UDEV_175 */
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
+		"ENV{DEVTYPE}!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n"
+		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{builtin}=\"usb_id\"\n"
+		/* ignore mass storage class having devices in mark-up */
+		"ENV{ID_USB_INTERFACES}==\"*:08*:*\", GOTO=\"libgphoto2_usb_end\"\n"
+		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
+		 * more rules . It will be completed in udev_begin_func() */
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
+
+		/* UDEV_201 ... regular stuff is done via hwdb, only scsi generic here. */
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
+		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{builtin}=\"usb_id\"\n"
+		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
+		 * more rules . It will be completed in udev_begin_func() */
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\"",
 	};
 	static const char * const usbcam_strings[] = {
 		/* UDEV_PRE_0_98 */
-		"SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\", ",
+		"SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\"",
 		/* UDEV_0_98 */
-		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ",
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\"",
+		/* UDEV_136 */
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_175 */
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_201 */
+		""
+	};
+	static const char * const usbdisk_strings[] = {
+		/* UDEV_PRE_0_98 */
+		"KERNEL==\"%s\", SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\"",
+		/* UDEV_0_98 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\"",
+		/* UDEV_136 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_175 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_201 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\""
 	};
 	udev_persistent_data_t *pdata;
 	pdata = calloc(1, sizeof(udev_persistent_data_t));
@@ -428,7 +476,8 @@ udev_parse_params (const func_params_t *params, void **data)
 			} else if (strcmp("group", key)==0) {
 				pdata->group = val;
 			} else if (strcmp("version", key)==0) {
-				if (gpi_string_to_enum(val, &(pdata->version),
+				unsigned int *ver = &pdata->version;
+				if (gpi_string_to_enum(val, ver,
 						       udev_version_t_map)) {
 					FATAL("Unrecognized udev version: \"%s\"", val);
 				}
@@ -443,10 +492,11 @@ udev_parse_params (const func_params_t *params, void **data)
 		}
 	}
 	if ((0==0)
-	    && pdata->mode == NULL 
-	    && pdata->group == NULL 
-	    && pdata->owner == NULL 
-	    && pdata->script == NULL) {
+	    && (pdata->mode == NULL)
+	    && (pdata->group == NULL)
+	    && (pdata->owner == NULL)
+	    && (pdata->script == NULL)
+	    && (pdata->version <= UDEV_0_98)) {
 		FATAL("Either <script> or <mode,group,owner> parameters must be given.");
 	}
 	if ((pdata->script != NULL) && (pdata->mode != NULL 
@@ -454,9 +504,12 @@ udev_parse_params (const func_params_t *params, void **data)
 					|| pdata->owner != NULL)) {
 		FATAL("The <script> parameter conflicts with the <mode,group,owner> parameters.");
 	}
+	if (pdata->version >= UDEV_201)
+		fprintf(stderr,"NOTE: You need to generate a hwdb too, this file just contains the scsi generic device entries.\n");
 
 	pdata->begin_string = begin_strings[pdata->version];
 	pdata->usbcam_string = usbcam_strings[pdata->version];
+	pdata->usbdisk_string = usbdisk_strings[pdata->version];
 }
 
 
@@ -479,12 +532,37 @@ udev_begin_func (const func_params_t *params, void **data)
 		printf ("# udev rules file for libgphoto2 devices (for udev %s version)\n",
 			version_str);
 		print_version_comment(stdout, "# ", "\n", NULL, "#\n");
-		printf (pdata->begin_string);
+		printf ("# this file is autogenerated, local changes will be LOST on upgrades\n");
+		printf ("%s", pdata->begin_string);
+
+		if (pdata->version == UDEV_136 || pdata->version == UDEV_175) {
+			if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
+				if (pdata->mode != NULL) {
+					printf("MODE=\"%s\", ", pdata->mode);
+				}
+				if (pdata->owner != NULL) {
+					printf("OWNER=\"%s\", ", pdata->owner);
+				}
+				if (pdata->group != NULL) {
+					printf("GROUP=\"%s\", ", pdata->group);
+				}
+			}
+
+			printf ("GOTO=\"libgphoto2_usb_end\"\n\n");
+		}
 	}
 	return 0;
 }
 
 
+static int
+udev_middle_func (const func_params_t *params, void **data)
+{
+	printf ("\nLABEL=\"libgphoto2_usb_end\"\n\n");
+	return 0;
+}
+
+
 static int
 udev_end_func (const func_params_t *params, void *data)
 {
@@ -507,7 +585,6 @@ udev_camera_func (const func_params_t *params,
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
-	int usb_vendor = 0, usb_product = 0;
 	int has_valid_rule = 0;
 	udev_persistent_data_t *pdata = (udev_persistent_data_t *) data;
 	ASSERT(pdata != NULL);
@@ -515,14 +592,14 @@ udev_camera_func (const func_params_t *params,
 	if (!(a->port & GP_PORT_USB))
 		return 0;
 
+	if (pdata->version == UDEV_201) return 0;
+
 	if (a->usb_vendor) { /* usb product id may be zero! */
 		class = 0;
 		subclass = 0;
 		proto = 0;
 		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
 			 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-		usb_vendor = a->usb_vendor;
-		usb_product = a->usb_product;
 	} else {
 		if (a->usb_class) {
 			class = a->usb_class;
@@ -537,8 +614,6 @@ udev_camera_func (const func_params_t *params,
 				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
 			else
 				proto = 0;
-			usb_vendor = 0;
-			usb_product = 0;
 		}
 	}
 
@@ -548,7 +623,11 @@ udev_camera_func (const func_params_t *params,
 
 	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
 		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
-			printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\", ", class, subclass, proto);
+			if (pdata->version == UDEV_136 || pdata->version == UDEV_175) {
+				printf("ENV{ID_USB_INTERFACES}==\"*:%02d%02d%02d:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\"", class, subclass, proto);
+			} else {
+				printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\"", class, subclass, proto);
+			}
 			has_valid_rule = 1;
 		} else {
 			if (class == 666) {
@@ -569,6 +648,12 @@ udev_camera_func (const func_params_t *params,
 		}
 	}
 	if (has_valid_rule != 0) {
+		if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
+			printf(", ENV{ID_MEDIA_PLAYER}=\"1\"");
+
+		if (pdata->script != NULL || pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL)
+			printf(", ");
+
 		if (pdata->script != NULL) {
 			printf("RUN+=\"%s\"\n", pdata->script);
 		} else if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
@@ -590,13 +675,154 @@ udev_camera_func (const func_params_t *params,
 			printf("\n");
 		} else {
 			printf("\n");
-			FATAL("udev_camera_func(): illegal branch");
+			if (pdata->version < UDEV_136)
+				FATAL("udev_camera_func(): illegal branch");
 		}
 	}
 	return 0;
 }
 
 
+static int
+udev_camera_func2 (const func_params_t *params, 
+		   const int i,
+		   const int total,
+		   const CameraAbilities *a,
+		   void *data)
+{
+	int has_valid_rule = 0;
+	udev_persistent_data_t *pdata = (udev_persistent_data_t *) data;
+	ASSERT(pdata != NULL);
+
+	if (a->port & GP_PORT_USB_DISK_DIRECT) {
+		printf (pdata->usbdisk_string, "sd[a-z]*",
+			a->usb_vendor, a->usb_product);
+		has_valid_rule = 1;
+	}
+	if (a->port & GP_PORT_USB_SCSI) {
+		printf (pdata->usbdisk_string, "sg[0-9]*",
+			a->usb_vendor, a->usb_product);
+		has_valid_rule = 1;
+	}
+	if (has_valid_rule != 0) {
+		if (pdata->script != NULL || pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL)
+			printf(", ");
+
+		if (pdata->script != NULL) {
+			printf("RUN+=\"%s\"\n", pdata->script);
+		} else if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
+			if (pdata->mode != NULL) {
+				printf("MODE=\"%s\"", pdata->mode);
+				if (pdata->owner != NULL || pdata->group != NULL) {
+					printf(", ");
+				}
+			}
+			if (pdata->owner != NULL) {
+				printf("OWNER=\"%s\"", pdata->owner);
+				if (pdata->group != NULL) {
+					printf(", ");
+				}
+			}
+			if (pdata->group != NULL) {
+				printf("GROUP=\"%s\"", pdata->group);
+			}
+			printf("\n");
+		} else {
+			printf("\n");
+			if (pdata->version < UDEV_136)
+				FATAL("udev_camera_func(): illegal branch");
+		}
+	}
+
+	return 0;
+}
+
+
+static int
+hwdb_begin_func (const func_params_t *params, void **data)
+{
+	fprintf(stderr,"NOTE: You should generate a udev rules file with udev-version 201\n");
+	fprintf(stderr,"or later to support cameras that use SCSI tunneling support, like\n");
+	fprintf(stderr,"various picture frames, Olympus remote control support.\n");
+	printf ("# hardware database file for libgphoto2 devices\n");
+	return 0;
+}
+
+
+static int
+hwdb_camera_func (const func_params_t *params, 
+		  const int i,
+		  const int total,
+		  const CameraAbilities *a,
+		  void *data)
+{
+	int flags = 0;
+	int class = 0, subclass = 0, proto = 0;
+	int usb_vendor = 0, usb_product = 0;
+	int has_valid_rule = 0;
+
+	if (!(a->port & GP_PORT_USB))
+		return 0;
+
+	if (a->usb_vendor) { /* usb product id may be zero! */
+		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+			 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+		usb_vendor = a->usb_vendor;
+		usb_product = a->usb_product;
+	} else {
+		if (a->usb_class) {
+			class = a->usb_class;
+			subclass = a->usb_subclass;
+			proto = a->usb_protocol;
+			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
+			if (subclass != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
+			else
+				subclass = 0;
+			if (proto != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
+			else
+				proto = 0;
+		}
+	}
+
+	printf ("\n# %s\n", a->model);
+
+	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
+		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
+			/* device class matcher ... */
+			/*printf("usb:v*p*d*dc%02ddsc%02dp%02d*\"\n GPHOTO2_DRIVER=PTP\n", class, subclass, proto);*/
+			/* but we need an interface class matcher, ptp is a interface */
+			printf("usb:v*ic%02disc%02dip%02d*\n GPHOTO2_DRIVER=PTP\n", class, subclass, proto);
+			has_valid_rule = 1;
+		} else {
+			if (class == 666) {
+				printf("# not working yet\n");
+			} else {
+				fprintf(stderr, "unhandled interface match flags %x\n", flags);
+			}
+		}
+	} else {
+		if (flags & GP_USB_HOTPLUG_MATCH_VENDOR_ID) {
+			if (strstr(a->library, "ptp"))
+				printf ("usb:v%04Xp%04X*\n GPHOTO2_DRIVER=PTP\n", usb_vendor, usb_product);
+			else
+				printf ("usb:v%04Xp%04X*\n GPHOTO2_DRIVER=proprietary\n", usb_vendor, usb_product);
+			has_valid_rule = 1;
+		} else {
+			fprintf (stderr, "Error: Trying to output device %d/%d with incorrect match flags.\n",
+				usb_vendor, usb_product
+			);
+		}
+	}
+	if (has_valid_rule != 0) {
+		printf(" ID_GPHOTO2=1\n");
+		if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
+			printf(" ID_MEDIA_PLAYER=1\n");
+	}
+	return 0;
+}
+
 static int
 empty_begin_func (const func_params_t *params, void **data)
 {
@@ -707,7 +933,7 @@ ddb_camera_func (const func_params_t *params,
 		 const CameraAbilities *a,
 		 void *data)
 {
-	const char *camlib_basename = basename(a->library);
+	const char *camlib_basename = path_basename(a->library);
 	int head_printed = 0;
 #define DELAYED_HEAD() \
 	do { \
@@ -842,9 +1068,14 @@ fdi_camera_func (const func_params_t *params,
 		if (a->usb_vendor) { /* usb product id might be 0! */
 			printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
 			printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
+			if (a->usb_vendor == 0x05ac) { /* Apple iPhone, PTP user. */
+				printf("     <match key=\"usb.interface.class\" int=\"6\">\n");
+				printf("      <match key=\"usb.interface.subclass\" int=\"1\">\n");
+				printf("       <match key=\"usb.interface.protocol\" int=\"1\">\n");
+			}
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER) {
 				printf("     <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
+				printf("     <addset key=\"info.capabilities\" type=\"strlist\">portable_audio_player</addset>\n");
 				printf("     <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
 				printf("     <merge key=\"portable_audio_player.type\" type=\"string\">mtp</merge>\n");
 				
@@ -852,8 +1083,8 @@ fdi_camera_func (const func_params_t *params,
 				printf("     <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
 			} else {
 				printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-				
+				printf("     <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
+
 				/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
 				if (NULL!=strstr(a->library,"ptp"))
 					printf("     <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
@@ -863,6 +1094,11 @@ fdi_camera_func (const func_params_t *params,
 			/* leave them here even for audio players */
 			printf("     <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
 			printf("     <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			if (a->usb_vendor == 0x05ac) { /* Apple iPhone */
+				printf("       </match>\n");
+				printf("      </match>\n");
+				printf("     </match>\n");
+			}
 			printf("    </match>\n");
 			printf("   </match>\n");
 			
@@ -871,7 +1107,7 @@ fdi_camera_func (const func_params_t *params,
 			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
 			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
 			printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-			printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("      <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
 			if (a->usb_class == 6) {
 				printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
 			} else {
@@ -1000,22 +1236,207 @@ fdi_device_end_func (const func_params_t *params, void *data)
 	return 0;
 }
 
+/* HTML output */
+struct html_comment {
+	char *name;
+	char *comment;
+};
+struct html_data {
+	int nrofcomments;
+	struct html_comment *comments;
+};
+
+static int
+html_begin_func (const func_params_t *params, void **data) {
+	FILE *f;
+	char buf[512];
+	struct html_data *hd;
+
+	printf("<!-- This part was generated by %s - - html -->\n",
+	       "libgphoto2 " ARGV0);
+	print_version_comment(stdout, "    | ", "\n", "<!--+\n", "    +-->\n");
+	printf("Number of supported cameras and media players: %d\n",params->number_of_cameras);
+	printf("<table border=1>\n");
+	printf("<tr>\n");
+	printf("   <th>Camera Model</th>\n");
+	printf("   <th>Additional Abilities</th>\n");
+	printf("   <th>Comments</th>\n");
+	printf("</tr>\n");
+
+	*data = hd = malloc(sizeof(*hd));
+	hd->nrofcomments = 0;
+	hd->comments = NULL;
+
+	f = fopen("comments.txt","r");
+	if (!f)
+		return 0;
+	while (fgets(buf,sizeof(buf),f)) {
+		char *s = strchr(buf,';');
+		if (!s)
+			continue;
+		*s = '\0';
+		if (hd->nrofcomments) {
+			hd->comments = realloc(hd->comments, (hd->nrofcomments+1)*sizeof(hd->comments[0]));
+		} else {
+			hd->comments = malloc(sizeof(hd->comments[0]));
+		}
+		hd->comments[hd->nrofcomments].name = strdup(buf);
+		hd->comments[hd->nrofcomments].comment = strdup(s+1);
+		hd->nrofcomments++;
+	}
+	fclose (f);
+	return 0;
+}
+
+static char*
+escape_html(const char *str) {
+	const char *s;
+	char *newstr, *ns;
+	int inc = 0;
+
+	s = str;
+	do {
+		s = strchr(s,'&');
+		if (s) {
+			inc+=strlen("&amp;");
+			s++;
+		}
+	} while (s);
+	/* FIXME: if we ever get a camera with <> or so, add escape code here */
+	newstr = malloc(strlen(str)+1+inc);
+	s = str; ns = newstr;
+	do {
+		char *x;
+		x = strchr(s,'&');
+		if (x) {
+			memcpy (ns, s, x-s);
+			ns += x-s;
+			memcpy (ns, "&amp;", strlen("&amp;"));
+			ns += strlen("&amp;");
+			s = x+1;
+		} else {
+			strcpy (ns, s);
+			break;
+		}
+	} while (1);
+	return newstr;
+}
+
+static int
+html_camera_func (
+	const func_params_t *params, 
+	const int i,
+	const int total,
+	const CameraAbilities *a,
+	void *data
+) {
+	char *m;
+	int j;
+	CameraOperation		op = a->operations;
+	struct html_data	*hd = data;
+
+	if (a->device_type != GP_DEVICE_STILL_CAMERA)
+		return 0;
+
+	printf ("<tr>\n");
+	m = escape_html (a->model);
+	printf (" <td>%s</td>", m); free (m);
+	printf (" <td>");
+	if (!op) printf ("&nbsp;");
+	if (op & GP_OPERATION_CAPTURE_IMAGE) {
+		printf ("Image Capture");
+		op &= ~GP_OPERATION_CAPTURE_IMAGE;
+		if (op) printf (", ");
+	}
+	if (op & GP_OPERATION_CAPTURE_PREVIEW) {
+		printf ("Liveview");
+		op &= ~GP_OPERATION_CAPTURE_PREVIEW;
+		if (op) printf (", ");
+	}
+	if (op & GP_OPERATION_CONFIG) {
+		printf ("Configuration");
+		op &= ~GP_OPERATION_CONFIG;
+		if (op) printf (", ");
+	}
+	if (op) {
+		printf ("Other Ops %x", op);
+	}
+	printf(" </td>");
+	printf(" <td>");
+	switch (a->status) {
+	case GP_DRIVER_STATUS_PRODUCTION: break;
+	case GP_DRIVER_STATUS_TESTING: printf("Testing (Beta)"); break;
+	case GP_DRIVER_STATUS_EXPERIMENTAL: printf("Experimental"); break;
+	case GP_DRIVER_STATUS_DEPRECATED: printf("Deprecated"); break;
+	}
+	/* read comments */
+	for (j=0;j<hd->nrofcomments;j++) {
+		if (!strcmp(a->model,hd->comments[j].name)) {
+			printf("%s",hd->comments[j].comment);
+			break;
+		}
+	}
+	if (j == hd->nrofcomments) printf(" &nbsp;");
+	printf(" </td>");
+	printf("</tr>\n");
+	return 0;
+}
+
+static int html_middle_func (
+	const func_params_t *params,
+	void **data
+) {
+	printf("</table><p>\n");
+	printf("Media Players that are supported by both libmtp and libgphoto2:<p/>\n");
+	printf("<table border=1>\n");
+	printf("<tr>\n");
+	printf("   <th>Media Player Model</th>\n");
+	printf("</tr>\n");
+	return 0;
+}
+
+static int
+html_camera2_func (
+	const func_params_t *params, 
+	const int i,
+	const int total,
+	const CameraAbilities *a,
+	void *data
+) {
+	char *m;
+
+	if (a->device_type != GP_DEVICE_AUDIO_PLAYER)
+		return 0;
+
+	m = escape_html (a->model);
+	printf ("<tr>\n");
+	printf (" <td>%s</td>", m); free (m);
+	printf("</tr>\n");
+	return 0;
+}
+
+
+static int html_end_func (
+	const func_params_t *params,
+	void *data
+) {
+	printf("</table>\n");
+	return 0;
+}
 
 /* time zero for debug log time stamps */
 struct timeval glob_tv_zero = { 0, 0 };
 
 static void
-debug_func (GPLogLevel level, const char *domain, const char *format,
-	    va_list args, void *data)
+debug_func (GPLogLevel level, const char *domain, const char *str,
+	    void *data)
 {
 	struct timeval tv;
 	gettimeofday (&tv,NULL);
-	fprintf (stderr, "%li.%06li %s(%i): ", 
-		 tv.tv_sec - glob_tv_zero.tv_sec, 
-		 (1000000 + tv.tv_usec - glob_tv_zero.tv_usec) % 1000000,
-		 domain, level);
-	vfprintf (stderr, format, args);
-	fprintf (stderr, "\n");
+	fprintf (stderr, "%li.%06li %s(%i): %s\n", 
+		 (long) (tv.tv_sec - glob_tv_zero.tv_sec), 
+		 (1000000L + tv.tv_usec - glob_tv_zero.tv_usec) % 1000000L,
+		 domain, level, str);
 }
 
 
@@ -1026,6 +1447,8 @@ typedef struct {
 	char *paramdescr;
 	begin_func_t begin_func;
 	camera_func_t camera_func;
+	middle_func_t middle_func;
+	camera_func_t camera_func2;
 	end_func_t end_func;
 } output_format_t;
 
@@ -1072,6 +1495,20 @@ iterate_camera_list (const int add_comments,
 		}
 	}
 
+	if (format->middle_func != NULL) {
+		format->middle_func(&params, &data);
+	}
+
+	if (format->camera_func2 != NULL) {
+		int i;
+		for (i = 0; i < number_of_cameras; i++) {
+			ret = gp_abilities_list_get_abilities (al, i, &a);
+			if (ret < GP_OK)
+				continue;
+			format->camera_func2(&params, i, number_of_cameras, &a, data);
+		}
+	}
+
 	if (format->end_func != NULL) {
 		format->end_func(&params, data);
 	}
@@ -1090,6 +1527,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 human_begin_func,
 	 human_camera_func,
+	 NULL,
+	 NULL,
 	 human_end_func
 	},
 	{"usb-usermap",
@@ -1100,6 +1539,8 @@ static const output_format_t formats[] = {
 	 "<NAME_OF_HOTPLUG_SCRIPT>",
 	 hotplug_begin_func,
 	 hotplug_camera_func,
+	 NULL,
+	 NULL,
 	 empty_end_func
 	},
 	{"hal-fdi",
@@ -1108,6 +1549,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 fdi_begin_func,
 	 fdi_camera_func,
+	 NULL,
+	 NULL,
 	 fdi_end_func
 	},
 	{"hal-fdi-device",
@@ -1116,26 +1559,57 @@ static const output_format_t formats[] = {
 	 NULL,
 	 fdi_device_begin_func,
 	 fdi_device_camera_func,
+	 NULL,
+	 NULL,
 	 fdi_device_end_func
 	},
 	{"udev-rules",
-	 "udev rules file either for pre-0.98 or 0.98 and later",
-	 "Put it into /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
+	 "udev rules file",
+	 "For modes \"pre-0.98\" and \"0.98\" (and later), put it into\n"
+	 "        /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
 	 "        or add script to run. This rule files also uses the\n"
 	 "        check-ptp-camera script included in libgphoto2 source. Either put it to\n"
 	 "        /lib/udev/check-ptp-camera or adjust the path in the generated rules file.\n"
-	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.",
+	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.\n"
+	 "        For mode \"136\" put it into /lib/udev/rules.d/40-libgphoto2.rules;\n"
+	 "        you can still use mode/owner/group, but the preferred mode of operation\n"
+	 "        is to use udev-extras for dynamic access permissions.\n"
+	"	  Available versions of the rule generator: 0.98, 136, 175, 201.\n",
 	 "[script <PATH_TO_SCRIPT>|version <version>|mode <mode>|owner <owner>|group <group>]*",
 	 udev_begin_func, 
 	 udev_camera_func,
+	 udev_middle_func,
+	 udev_camera_func2,
 	 udev_end_func
 	},
+	{"hwdb",
+	 "hardware database file",
+	 "Put it into /usr/lib/udev/hwdb.d/20-gphoto.conf.\n",
+	 NULL,
+	 hwdb_begin_func,
+	 hwdb_camera_func,
+	 NULL,
+	 NULL,
+	 empty_end_func
+	},
+	{"html",
+	 "HTML table file for gphoto.org website",
+	 "Paste it into /proj/libgphoto2/support.php",
+	 NULL,
+	 html_begin_func,
+	 html_camera_func,
+	 html_middle_func,
+	 html_camera2_func,
+	 html_end_func
+	},
 	{"idlist",
 	 "list of IDs and names",
 	 "grep for an ID to find the device name",
 	 NULL,
 	 empty_begin_func, 
 	 idlist_camera_func,
+	 NULL,
+	 NULL,
 	 empty_end_func
 	},
 #ifdef ENABLED_GP2DDB
@@ -1145,11 +1619,12 @@ static const output_format_t formats[] = {
 	 NULL,
 	 ddb_begin_func,
 	 ddb_camera_func,
+	 NULL,
+	 NULL,
 	 ddb_end_func
 	},
 #endif
-	{NULL, NULL, NULL, NULL, 
-	 NULL, NULL, NULL}
+	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
 
